/*
 * Copyright (C) 2000-2002 Red Hat, Inc.
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "../config.h"
#include <libintl.h>
#include <locale.h>
#include <pwd.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>

#include <glade/glade-xml.h>
#include <gtk/gtk.h>
#include <krb5.h>
#include "principals.h"
#include "gui_utils.h"
#include "tickettray.h"

#ifndef KRB5_SUCCESS
#define KRB5_SUCCESS 0
#endif

#define _(String) dgettext(PACKAGE, String)

char *principal = NULL, *cachefile = NULL;
krb5_context context;


static void flagscheckbox_set_state(GValue * value, GladeXML * xml, const gchar * widgetname)
{
	gboolean onoff;
	GtkWidget *widget;
	onoff = g_value_get_boolean(value);
	widget = glade_xml_get_widget(xml, widgetname);
	g_object_set(G_OBJECT(widget), "active",onoff,NULL);
	g_value_unset(value);

}
/*whatever pointer is for the row-activated signal..*/
static void on_principal_selected(GtkTreeView * treeview, GtkTreePath * arg1, gpointer *whatever, gpointer user_data)
{
	GtkTreeIter iter;
	GValue value;
	GtkTreeModel *model;
	GtkWidget *widget;
	const gchar *tmp1;
	GladeXML *xml = user_data;

	memset(&value, '\0', sizeof(value));
	model = gtk_tree_view_get_model(treeview);
	if (!gtk_tree_model_get_iter(model, &iter, arg1)) {
		return;
	}


	widget = glade_xml_get_widget(xml, "servernameentry");
	gtk_tree_model_get_value(model, &iter, 0, &value);
	tmp1 = g_value_get_string(&value);
	tmp1 = tmp1 ? tmp1 : "";
	gtk_entry_set_text(GTK_ENTRY(widget), tmp1);
	g_value_unset(&value);

	widget = glade_xml_get_widget(xml, "clientnameentry");
	gtk_tree_model_get_value(model, &iter, 1, &value);
	tmp1 = g_value_get_string(&value);
	tmp1 = tmp1 ? tmp1 : "";
	gtk_entry_set_text(GTK_ENTRY(widget), tmp1);
	g_value_unset(&value);
	//.. what is a loop ?
	gtk_tree_model_get_value(model, &iter, 2, &value);
	flagscheckbox_set_state(&value, xml, "forwardablecheckbutton");

	gtk_tree_model_get_value(model, &iter, 3, &value);
	flagscheckbox_set_state(&value, xml, "forwardedcheckbutton");

	gtk_tree_model_get_value(model, &iter, 4, &value);
	flagscheckbox_set_state(&value, xml, "proxiablecheckbutton");

	gtk_tree_model_get_value(model, &iter, 5, &value);
	flagscheckbox_set_state(&value, xml, "proxycheckbutton");

	gtk_tree_model_get_value(model, &iter, 6, &value);
	flagscheckbox_set_state(&value, xml, "postdatecheckbutton");

	gtk_tree_model_get_value(model, &iter, 7, &value);
	flagscheckbox_set_state(&value, xml, "postdatedcheckbutton");

	gtk_tree_model_get_value(model, &iter, 8, &value);
	flagscheckbox_set_state(&value, xml, "invalidcheckbutton");

	gtk_tree_model_get_value(model, &iter, 9, &value);
	flagscheckbox_set_state(&value, xml, "renewablecheckbutton");

	gtk_tree_model_get_value(model, &iter, 10, &value);
	flagscheckbox_set_state(&value, xml, "initialcheckbutton");

	gtk_tree_model_get_value(model, &iter, 11, &value);
	flagscheckbox_set_state(&value, xml, "preauthcheckbutton");

	gtk_tree_model_get_value(model, &iter, 12, &value);
	widget = glade_xml_get_widget(xml, "starttimeentry");
	tmp1 = g_value_get_string(&value);
	tmp1 = tmp1 ? tmp1 : "";
	gtk_entry_set_text(GTK_ENTRY(widget), tmp1);
	g_value_unset(&value);

	gtk_tree_model_get_value(model, &iter, 13, &value);
	widget = glade_xml_get_widget(xml, "lifetimeentry");
	tmp1 = g_value_get_string(&value);
	tmp1 = tmp1 ? tmp1 : "";
	gtk_entry_set_text(GTK_ENTRY(widget), tmp1);
	g_value_unset(&value);

	gtk_tree_model_get_value(model, &iter, 14, &value);
	widget = glade_xml_get_widget(xml, "renewabletimeentry");
	tmp1 = g_value_get_string(&value);
	tmp1 = tmp1 ? tmp1 : "";
	gtk_entry_set_text(GTK_ENTRY(widget), tmp1);
	g_value_unset(&value);
}

static void  on_apply_clicked (GtkButton *button,gpointer user_data)
{
	GError *error = 0;
	gint status;
	GtkWidget *widget;
	GladeXML *xml = user_data;
	widget = glade_xml_get_widget(xml, "mainWindow");
	if(!g_spawn_command_line_sync ("gkinit",NULL,NULL,&status,&error)){
		gchar *err = g_strdup_printf(_("Error staring authentication application:\n\n%s"),error->message);
		g_clear_error(&error);
		show_error(widget,err);
		g_free(err);
	} else {
		GtkListStore *liststore;
		widget = glade_xml_get_widget(xml, "treeview");
		liststore = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(widget)));

		list_principals(widget,liststore);
	}

}

static gboolean view_onButtonPressed(GtkWidget * treeview, GdkEventButton * event, gpointer userdata)
{
	if (event->type == GDK_BUTTON_PRESS) {
		GtkTreePath *path;

		if (!gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(treeview), event->x, event->y, &path, NULL, NULL, NULL)) {
			return FALSE;
		}
		/* click with the left mouse button? */
		if (event->button == 1) {
			on_principal_selected(GTK_TREE_VIEW(treeview), path, NULL,userdata);
		}

		gtk_tree_path_free(path);
	}

	return FALSE;		/* we pretend we did not handle this */
}

static gboolean destroy_mainwindow(GtkWidget * widget, GdkEventAny * event, GladeXML * xml)
{
	gtk_widget_destroy(widget);
	g_object_unref(G_OBJECT(xml));
	set_mainwindow_destroyed(TRUE);
	return TRUE;

}

void init_and_show_gui()
{
	GladeXML *xml;
	GtkWidget *widget;
	GtkTreeViewColumn *column;
	GtkCellRenderer *renderer;
	krb5_principal princ;
	krb5_ccache ccache;
	GtkListStore *liststore = NULL;
	int ret;
	/* Load the GUI. */
	xml = glade_xml_new(DATADIR "/" PACKAGE "/gktickets.glade", "mainWindow", PACKAGE);
	if (xml == NULL) {
		xml = glade_xml_new("gktickets.glade", "mainWindow", PACKAGE);
	}
	if (xml == NULL) {
		show_error(NULL,_("Error initializing GUI!"));
		exit(1);
	}
	ret = krb5_cc_resolve(context, cachefile, &ccache);
	if (ret != KRB5_SUCCESS) {
		gchar *p;
		p = g_strdup_printf(_("Error resolving ticket cache name:\n%s"), error_message(ret));
		show_error(NULL,p);
		g_free(p);
		exit(1);
	}
	/* Get the default principal's name and set it in the right label. */
	ret = krb5_cc_get_principal(context, ccache, &princ);
	if (ret == KRB5_SUCCESS) {
		gchar *tmp;
		ret = krb5_unparse_name(context, princ, &tmp);
		if (ret == KRB5_SUCCESS) {
			if (principal != NULL) {
				g_free(principal);
			}
			principal = g_strdup(tmp);
			krb5_free_unparsed_name(context, tmp);
			krb5_free_principal(context, princ);
		}
	} else {
		struct passwd *pwd;
		setpwent();
		pwd = getpwuid(getuid());
		if (principal) {
			g_free(principal);
		}
		principal = (pwd != NULL) ? g_strdup(pwd->pw_name): g_strdup("");
		endpwent();
	}
	widget = glade_xml_get_widget(xml, "theprincipalentry");
	if (GTK_IS_ENTRY(widget)) {
		gtk_entry_set_text(GTK_ENTRY(widget), principal);
	}

	/* Set up the list store for use by the TreeView in the dialog. */
	widget = glade_xml_get_widget(xml, "treeview");
	if (GTK_IS_TREE_VIEW(widget)) {
		liststore = gtk_list_store_new(15,
					       G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, 
					       G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, 
					       G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, 
					       G_TYPE_STRING,G_TYPE_STRING, G_TYPE_STRING);

		gtk_tree_view_set_model(GTK_TREE_VIEW(widget), GTK_TREE_MODEL(liststore));

		column = gtk_tree_view_column_new();
		renderer = gtk_cell_renderer_text_new();
		gtk_tree_view_column_pack_start(column, renderer, TRUE);
		gtk_tree_view_column_set_title(column, _("Principal"));
		gtk_tree_view_column_set_attributes(column, renderer, "text", 0, NULL);
		gtk_tree_view_insert_column(GTK_TREE_VIEW(widget), column, 0);
		g_signal_connect(G_OBJECT(widget), "button-press-event", G_CALLBACK(view_onButtonPressed), xml);
		/* row-activated seems ok for catching key navigations.. */
		g_signal_connect(G_OBJECT(widget), "row-activated", G_CALLBACK(on_principal_selected), xml);

	}
	krb5_cc_close(context, ccache);

	/* Refresh the creds list. */
	list_principals(glade_xml_get_widget(xml, "mainWindow"),liststore);

	/* Show the main window. */
	widget = glade_xml_get_widget(xml, "mainWindow");
	g_signal_connect(G_OBJECT(widget), "delete_event", G_CALLBACK(destroy_mainwindow), xml);
	widget = glade_xml_get_widget(xml, "applybutton");
	g_signal_connect(G_OBJECT(widget), "clicked", G_CALLBACK(on_apply_clicked), xml);
	gtk_widget_show_all(widget);

}

int main(int argc, char **argv)
{
	int ret;

	/* Initialize the locale settings. */
	setlocale(LC_ALL, "");
	bindtextdomain(PACKAGE, GNOMELOCALEDIR);
	textdomain(PACKAGE);

	/* Initialize GTK+. */
	g_random_set_seed((guint32) getpid());
	gtk_init(&argc, &argv);
	/* Initialize Kerberos. */
	initialize_krb5_error_table();
	if ((ret = krb5_init_context(&context)) != KRB5_SUCCESS) {
		gchar *p;
		p = g_strdup_printf(_("Error initializing Kerberos:\n%s"), error_message(ret));
		show_error(NULL,p);
		g_free(p);
		return 1;
	}

	cachefile = g_strdup(krb5_cc_default_name(context));
	ret = krb5_cc_set_default_name(context, cachefile);
	if (ret != KRB5_SUCCESS) {
		gchar *p;
		p = g_strdup_printf(_("Error setting default cache name:\n%s"), error_message(ret));
		show_error(NULL,p);
		g_free(p);
		return 1;
	}

	init_and_show_gui();
	setupTrayIcon();

	gtk_main();
	//make valgrind happy
	krb5_free_context(context);

	return 0;
}

