#include "../config.h"
#include <libintl.h>
#include <locale.h>
#include "principals.h"
#include "gui_utils.h"
#include "common.h"

#ifndef KRB5_SUCCESS
#define KRB5_SUCCESS 0
#endif

#define _(String) dgettext(PACKAGE, String)


void list_principals(GtkWidget *mainwindow,GtkListStore * liststore)
{
	krb5_cc_cursor cursor;
	krb5_creds creds;
	krb5_ccache ccache;
	GtkTreeIter iter;
	char buf[LINE_MAX];
	int ret ;

	ret = krb5_cc_resolve(context, cachefile, &ccache);
	if (ret != KRB5_SUCCESS) {
		char *p = g_strdup_printf(_("Error resolving cache name:\n%s"),error_message(ret));
		show_error(mainwindow,p);
		g_free(p);
		return;
	}

	gtk_list_store_clear(liststore);
	ret = krb5_cc_start_seq_get(context, ccache, &cursor);
	if (ret == KRB5_SUCCESS) {
		while (krb5_cc_next_cred(context, ccache, &cursor, &creds) == KRB5_SUCCESS) {
			char *p = NULL;
			krb5_timestamp starttime;

			gtk_list_store_append(liststore, &iter);
			krb5_unparse_name(context, creds.server, &p);
			gtk_list_store_set(liststore, &iter, 0, p, -1);
			g_free(p);
			krb5_unparse_name(context, creds.client, &p);
			gtk_list_store_set(liststore, &iter, 1, p, -1);
			g_free(p);
			gtk_list_store_set(liststore, &iter, 2, creds.ticket_flags & TKT_FLG_FORWARDABLE, -1);
			gtk_list_store_set(liststore, &iter, 3, creds.ticket_flags & TKT_FLG_FORWARDED, -1);
			gtk_list_store_set(liststore, &iter, 4, creds.ticket_flags & TKT_FLG_PROXIABLE, -1);
			gtk_list_store_set(liststore, &iter, 5, creds.ticket_flags & TKT_FLG_PROXY, -1);
			gtk_list_store_set(liststore, &iter, 6, creds.ticket_flags & TKT_FLG_MAY_POSTDATE, -1);
			gtk_list_store_set(liststore, &iter, 7, creds.ticket_flags & TKT_FLG_POSTDATED, -1);
			gtk_list_store_set(liststore, &iter, 8, creds.ticket_flags & TKT_FLG_INVALID, -1);
			gtk_list_store_set(liststore, &iter, 9, creds.ticket_flags & TKT_FLG_RENEWABLE, -1);
			gtk_list_store_set(liststore, &iter, 10, creds.ticket_flags & TKT_FLG_INITIAL, -1);
			gtk_list_store_set(liststore, &iter, 11, creds.ticket_flags & TKT_FLG_PRE_AUTH, -1);

			starttime = (creds.times.starttime != 0) ? creds.times.starttime : creds.times.authtime;

			if (krb5_timestamp_to_string(starttime, buf, sizeof(buf)) == KRB5_SUCCESS) {
				gtk_list_store_set(liststore, &iter, 12, buf, -1);
			}
			if (krb5_timestamp_to_string(creds.times.endtime, buf, sizeof(buf)) == KRB5_SUCCESS) {
				gtk_list_store_set(liststore, &iter, 13, buf, -1);
			}
			if (creds.times.renew_till != 0) {
				if (krb5_timestamp_to_string(creds.times.renew_till, buf, sizeof(buf)) == KRB5_SUCCESS) {
					gtk_list_store_set(liststore, &iter, 14, buf, -1);
				}
			}

			krb5_free_cred_contents(context, &creds);
		}
		
		krb5_cc_end_seq_get(context, ccache, &cursor);
	} else if (ret != KRB5_FCC_NOFILE) {
		char *p = g_strdup_printf(_("Error getting cached principals:\n%s"),error_message(ret));
		show_error(mainwindow,p);
		g_free(p);
	}

	krb5_cc_close(context, ccache);
}

ticket_status check_initial_ticket(guint32 *endtime)
{
	krb5_cc_cursor cursor;
	krb5_creds creds;
	krb5_ccache ccache;
	krb5_principal princ;
	krb5_creds tgt;
	krb5_data *client_realm;
	gboolean found = FALSE;
	ticket_status status;
	int ret ;

	status = NO_TICKET;
	*endtime = 0;
	memset(&tgt,'\0',sizeof(tgt));
	ret = krb5_cc_resolve(context, cachefile, &ccache);
	if (ret != KRB5_SUCCESS) {
		return status;
	}

	ret = krb5_cc_get_principal(context, ccache, &princ);
	if (ret != KRB5_SUCCESS) {
		krb5_cc_close(context, ccache);
		return status;
	}

	client_realm = krb5_princ_realm (context, princ);
	ret = krb5_build_principal (context, &tgt.server,client_realm->length,client_realm->data, KRB5_TGS_NAME, client_realm->data,NULL);
        if (ret != KRB5_SUCCESS) {
		krb5_free_principal(context, princ);
		krb5_cc_close(context, ccache);
		return status;
	}

	tgt.client = princ;

	ret = krb5_cc_retrieve_cred (context, ccache, 0, &tgt, &creds);
        if (ret != KRB5_SUCCESS) {
		//krb5_free_principal(context, princ); //line below takes care
		krb5_free_cred_contents(context, &tgt);
		krb5_cc_close(context, ccache);
		return status;
	}

	 
	status = TICKET_OK;
	*endtime = (guint32)creds.times.endtime;
	krb5_free_cred_contents(context, &creds);
	krb5_free_cred_contents(context, &tgt);
	//krb5_free_principal(context, princ);
	krb5_cc_close(context, ccache);

	return status;	
}
